/*
 *   Copyright (c) International Business Machines  Corp.2005
 *              
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *       
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 ***************************************************************************
 * File: stress1_test.c
 *
 * Description: This test stresses auditd by making repeated syscalls and
 *              rotating the log files repeatedly.  The number of syscalls
 *              can be increased in the loop to as many as you wish.  The
 *              disk will not fill up because the log files are rotated and
 *              reused.  The intent of this test is not to check for lost
 *              records; see noloss_test.c and the disk handling tests.  
 * 
 * Total Tests: 1M passes through loop; can be altered.
 *
 * Test Name: stress1_test
 *
 * Test Assertion & Strategy:
 *
 *              This file is to be run as root and will cause successful
 *              syscall records.  stress2_test should be run in another
 *              window as ealuser at the same time as this test.  
 *              stress2_test will result in unsuccessful syscall audit
 *              records. Stress3_test.sh should also be run at the same
 *              time; it creates and su's to another user who runs
 *              stress3_test which also results in successful calls. Start 
 *              stress1 first; it contains the setup for all tests. 
 *      
 *              At the end of the test you should visually verify that the
 *              log files contain records for uid's 0 and 560 with 
 *              "success=yes" and records for uid 500 with "success=no".  
 *              If any of these is missing the test has failed.
 *
 *              If this test terminates prematurely, you may need to
 *              restore the original /etc/auditd.conf file from
 *              /etc/auditd.conf.save. Premature termination caused by a
 *              problem with the system must be considered a failure and
 *              investigated. When the test completes normally, a completion
 *              message will display.
 *
 * History:
 *
 * FLAG   DATE     NAME               DESCRIPTION
 *       4/19/05  krisw@us.ibm.com  Created this test.
 *
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/fcntl.h>
#include <asm/unistd.h>
#include <signal.h>

mode_t mode;                    /* mode for directory creation*/
int rc;
void cleanup();
void setup();

int main(int ac, char **av)
{
   int lc;                      /* loop counter */
   char *msg;                   /* message returned from parse_opts */
   char dirname[40] ="stress1_dir";

   setup();        /* Call global setup for test */

/* Exercise the rules  1,000,000 times. */

     for (lc = 0; lc < 1000000; lc++) {
        printf("THIS IS LOOP %d \n", lc);
        syscall(__NR_mkdir,dirname,mode);
        syscall(__NR_chmod,dirname,mode);
        syscall(__NR_rmdir,dirname);
     }

     cleanup();
     printf("The stress1_test has completed normally. \n");
     return(0);
}

/* setup() - performs all ONE TIME setup for this test */

void
setup(void)
{

/* Initialize mode. */
         mode = (S_IRWXU | S_IRWXG | S_IRWXO);

/* Clear any existing rules. */
   system("auditctl -D");

/* Save the auditd.conf file to another name. */
        system("mv -f /etc/auditd.conf /etc/auditd.conf.save");

/* Copy the test configuration file to auditd.conf. NOTE: This ensures
   we know exactly what is in the file at the start of the test. */
        system("cp ./stress.conf /etc/auditd.conf");

/* Stop auditd and remove any /var/log/audit/*.log files.*/
         system("service auditd stop");
         system("rm -f /var/log/audit/audit*");

/* Restart auditd */
   rc = (system("service auditd start"));
   if (rc != 0) {
     printf("Auditd start failed.");
   } else {

/* Create rules using auditctl. */
     system("auditctl -a entry,always -S mkdir");
     system("auditctl -a entry,always -S chmod");
     system("auditctl -a entry,always -S rmdir");
     system("auditctl -a exit,always -S mkdir");
     system("auditctl -a exit,always -S chmod");
     system("auditctl -a exit,always -S rmdir");
   }
}

/* cleanup() - performs cleanup for this test */
void
cleanup(void)
{

/* Stop auditd */
   system("service auditd stop");

/* Clear any existing rules. */
   system("auditctl -D");


/* Restore the /etc/auditd.conf file. */
   system("mv -f /etc/auditd.conf.save /etc/auditd.conf");

}

